unit BaseTypes;

{$mode objfpc}{$H+}{$A8}
{$PACKRECORDS DEFAULT}
{$MODESWITCH AdvancedRecords+}

interface
uses
  Classes, SysUtils, SyncObjs;

{$PACKRECORDS 8}
type TNumberUtil = packed record
  constructor Create(Value:  int64);
  constructor Create(Value: uint64);
  function DwordToHex(LeadingZeros: Boolean): AnsiString;
  Case Mode: int8 of
    0: (S8_0,S8_1,S8_2,S8_3,S8_4,S8_5,S8_6,S8_7  : int8);
    1: (U8_0,U8_1,U8_2,U8_3,U8_4,U8_5,U8_6,U8_7 : uint8);
    2: (S16_0,S16_1,S16_2,S16_3 : int16);
    3: (U16_0,U16_1,U16_2,U16_3 : uint16);
    4: (S32_0,S32_1 : int32);
    5: (U32_0,U32_1 : uint32);
    6: (S64_0 : int64);
    7: (U64_0 : uint64);
    8: (UP_0 : PtrUInt);
    9: (SP_0 : PtrInt);

end;
{$PACKRECORDS DEFAULT}

{%Region TLocker}
type TLocker = class
  private
    var _CS       : TCriticalSection;
    var _Count    : Int32;
  public
    constructor Create;
    destructor Destroy; override;

end;
{%Endregion}
type TDummyClass = class
  ModuleLocker: TLocker;
  constructor Create;
  destructor Destroy; override;
end;
implementation
var
 Dummy : TDummyClass;


constructor TNumberUtil.Create(Value:  int64);
begin
  S64_0:= Value;
end;
constructor TNumberUtil.Create(Value: uint64);
begin
  U64_0:= Value;
end;

function TNumberUtil.DwordToHex(LeadingZeros: Boolean): AnsiString;
var
  _tmp : AnsiString;
  _len : Int32;
  _val : UInt16;
begin
  //if LeadingZeros then

  //begin
    if U16_0=U8_0 then _len:= 1;
  //end;

  SetLength(_tmp, _Len + _Len);
  _val:=U16_0;
  {$IFDEF ENDIAN_LITTLE}
  _val:=SwapEndian(_val);
  {$ENDIF}
  BinToHex(Pointer(PtrUint(@_val)-_len+2) ,@_tmp[1],_len);
  Result:= _tmp;
end;



{%Region TLocker}
constructor TLocker.Create;
begin
  if not Assigned(_CS) then
  begin
    _CS:= TCriticalSection.Create;
    _Count := 0;
  end;
end;
destructor TLocker.Destroy;
begin
  if Assigned(_CS) And (_Count>=0) then
  begin
    _CS.Acquire;
    if _Count>=0 then
    begin
      _Count:=-1;
      _CS.Release;
      FreeAndNil(_CS);
    end;
  end;
end;

constructor TDummyClass.Create;
begin
  inherited Create;
  begin
    ModuleLocker:= TLocker.Create;
  end;
end;
destructor TDummyClass.Destroy;
begin
  begin
    FreeAndNil(ModuleLocker);
  end;
  inherited Destroy;
end;
{%Region InitFinal}
procedure _Initialize();
begin
  If not Assigned(Dummy) then Dummy := TDummyClass.Create;
end;
procedure _Finalize();
begin
  If Assigned(Dummy)  then FreeAndNil(Dummy);
end;
initialization
  _Initialize;
finalization
  _Finalize;
{%Endregion}
end.

